package com.ruoyi.jjh.ent.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.jjh.declaration.service.IBmsDeclarationRecordsService;
import com.ruoyi.jjh.ent.entity.JSmartReminders;
import com.ruoyi.jjh.ent.entity.request.JSmartRemindersDq;
import com.ruoyi.jjh.ent.entity.request.JSmartRemindersNDq;
import com.ruoyi.jjh.ent.mapper.JSmartRemindersMapper;
import com.ruoyi.jjh.ent.service.JSmartRemindersService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDate;

/**
 * 智能提醒（j_smart_reminders）表业务实现层
 *
 * @author du
 * @since 2024/7/1 13:33
 */
@Service
public class JSmartRemindersServiceImpl extends ServiceImpl<JSmartRemindersMapper, JSmartReminders> implements JSmartRemindersService {



    @Resource
    private IBmsDeclarationRecordsService bmsDeclarationRecordsService;
    /**
     * 分页查询所有数据
     *
     * @param page            分页对象
     * @param jSmartReminders 查询实体
     * @return 所有数据
     */
    @Override
    public Page<JSmartReminders> page(Page<JSmartReminders> page, JSmartReminders jSmartReminders) {
        return baseMapper.page(page, jSmartReminders);
    }

    /**
     * 新增定期提醒数据
     *
     * @param jSmartRemindersDq 实体对象
     * @return 新增结果
     */
    @Override
    public Long add(JSmartRemindersDq jSmartRemindersDq) {
        JSmartReminders jSmartReminders = new JSmartReminders();
        BeanUtil.copyProperties(jSmartRemindersDq, jSmartReminders);
        if (LocalDate.now().isAfter(jSmartReminders.getAlertTime().toLocalDate())
                ||LocalDate.now().equals(jSmartReminders.getAlertTime().toLocalDate())) {
            throw new ServiceException("提醒时间需设置为今天之后!");
        }
        if (jSmartReminders.getAlertManner() == 2) {
            throw new ServiceException("不可新增不定期提醒!");
        }
        save(jSmartReminders);
        return jSmartRemindersDq.getId();
    }

    /**
     * 修改定期提醒数据
     *
     * @param jSmartRemindersDq 实体对象
     * @return 修改结果
     */
    @Override
    public Long updateDq(JSmartRemindersDq jSmartRemindersDq) {
        JSmartReminders jSmartReminders = new JSmartReminders();
        BeanUtil.copyProperties(jSmartRemindersDq, jSmartReminders);
        if (LocalDate.now().isAfter(jSmartReminders.getAlertTime().toLocalDate())
                ||LocalDate.now().equals(jSmartReminders.getAlertTime().toLocalDate())) {
            throw new ServiceException("提醒时间需设置为今天之后!");
        }
        if (jSmartReminders.getAlertManner() == 2) {
            throw new ServiceException("参数错误");
        }
        updateById(jSmartReminders);
        return jSmartRemindersDq.getId();
    }

    /**
     * 修改不定期提醒数据
     *
     * @param ndq 实体对象
     * @return 修改结果
     */
    @Override
    public Long updateNdq(JSmartRemindersNDq ndq) {
        JSmartReminders jSmartReminders = new JSmartReminders();
        BeanUtil.copyProperties(ndq, jSmartReminders);
        if (jSmartReminders.getAlertManner() == 1) {
            throw new ServiceException("参数错误");
        }
        updateById(jSmartReminders);
        return ndq.getId();
    }

//    /**
//     * 企业端首页智能提醒查询所有数据
//     *
//     * @return 所有数据
//     */
//    @Override
//    public List<SmartGetDeclaration> selectWorkAll() {

        //获取企业的不定时提醒
//        企业端
//        JSmartReminders qy = getById(1);
//        String userName = null;
//        try {
//            userName = SecurityUtils.getUsername();
//        } catch (Exception ignored) {
//        }
        //查询当前企业申报记录，对应的申报模板不是长期有效并且申报记录未填的，
//        List<SmartGetDeclaration> bmsDeclarationRecords = bmsDeclarationRecordsService.allNoLimited(userName);
//        //如果当前时间在该申报模板的结束时间前自定义的提前天数之前，就去掉
//        bmsDeclarationRecords.removeIf(x -> LocalDateTime.now().isBefore(x.getEndTime().minusDays(qy.getDaysAdvance())));
//        bmsDeclarationRecords.forEach(y->{
//            y.setDaysAdvance(qy.getDaysAdvance());
//            y.setAlertTime(y.getEndTime().minusDays(qy.getDaysAdvance()));
//            y.setAlertContent(qy.getAlertContent());
//        });
//        //获取企业的定时提醒
//        List<JSmartReminders> list = lambdaQuery().eq(JSmartReminders::getAlertRecipients, 1).eq(JSmartReminders::getAlertManner, 1).list();
//        list.forEach(x->{
//            if(LocalDateTime.now().isAfter(x.getAlertTime())){
//                SmartGetDeclaration sd = new SmartGetDeclaration();
//                sd.setAlertContent(x.getAlertContent());
//                sd.setAlertTime(x.getAlertTime());
//                bmsDeclarationRecords.add(sd);
//            }
//        });
//        bmsDeclarationRecords.sort(Comparator.comparing(entity -> Math.abs(ChronoUnit.MINUTES.between(LocalDateTime.now(), entity.getAlertTime()))));
//        return bmsDeclarationRecords;
//        return null;
//    }
}

